@@ -59,11 +59,11 @@ SQLThread::~SQLThread()
 void SQLThread::run(void)
 {
 	const char *opt;
-	long err;
+	SQLINTEGER err;
         SQLCHAR stat[10];
         SQLCHAR errmsg[128];
         SQLSMALLINT mlen, col, cols;
-        SQLINTEGER rowcnt;
+        long int rowcnt;
 	char buf[1024];
         char nbuf[12];
 	unsigned row = 0, len = 0;
