@@ -42,7 +42,7 @@ using namespace ost;
 
 static long tens[] = {1, 10, 100, 1000, 10000, 100000, 1000000, 10000000};
 
-long ScriptMethods::getRealValue(double d, unsigned prec)
+long ScriptInterp::getRealValue(double d, unsigned prec)
 {
 	char buf[20];
 	char *cp;
@@ -66,7 +66,7 @@ long ScriptMethods::getRealValue(double d, unsigned prec)
 	return rval + atol(lval);
 }
 
-long ScriptMethods::getIntValue(const char *text, unsigned prec, ScriptProperty *p) 
+long ScriptInterp::getIntValue(const char *text, unsigned prec, ScriptProperty *p) 
 {
 	Fun *fun = ifun;
 	unsigned count;
@@ -138,22 +138,22 @@ long ScriptMethods::getIntValue(const char *text, unsigned prec, ScriptProperty
 	return rval;
 }	
 
-double ScriptMethods::getDouble(long value, unsigned prec)
+double ScriptInterp::getDouble(long value, unsigned prec)
 {
         return (double)value / (double)tens[prec];
 }
 
-long ScriptMethods::getInteger(long value, unsigned prec)
+long ScriptInterp::getInteger(long value, unsigned prec)
 {
 	return value / tens[prec];
 }
 
-long ScriptMethods::getTens(unsigned prec)
+long ScriptInterp::getTens(unsigned prec)
 {
 	return tens[prec];
 }
 
-int ScriptMethods::numericExpression(long *vals, int max, unsigned prec, ScriptProperty *p)
+int ScriptInterp::numericExpression(long *vals, int max, unsigned prec, ScriptProperty *p)
 {
 	char estack[32];
 	unsigned esp = 0;
@@ -297,7 +297,7 @@ int ScriptMethods::numericExpression(long *vals, int max, unsigned prec, ScriptP
 		return -1;
 }
 
-bool ScriptMethods::conditionalExpression(void)
+bool ScriptInterp::conditionalExpression(void)
 {
 #ifdef	HAVE_REGEX_H
 	regex_t *regex;
@@ -310,6 +310,8 @@ bool ScriptMethods::conditionalExpression(void)
 	char n1[12], n2[12];
 	size_t l1, l2;
 	long ival;
+	time_t now;
+	Array *a;
 
 	// no first parm, invalid
 
@@ -340,8 +342,20 @@ bool ScriptMethods::conditionalExpression(void)
 		op = v1;
 	else if(!stricmp(v1, "-defined") || !stricmp(v1, "!defined"))
 		op = v1;
+	else if(!stricmp(v1, "-const") || !stricmp(v1, "!const"))
+		op = v1;
+	else if(!stricmp(v1, "-expired") || !stricmp(v1, "!expired"))
+		op = v1;
+	else if(!stricmp(v1, "-modify") || !stricmp(v1, "!modify"))
+		op = v1;
+	else if(!stricmp(v1, "-array") || !stricmp(v1, "!array"))
+		op = v1;
+        else if(!stricmp(v1, "-queue") || !stricmp(v1, "!queue"))
+                op = v1;
 	else if(!stricmp(v1, "-empty") || !stricmp(v1, "!empty"))
 		op = v1;
+        else if(!stricmp(v1, "-active") || !stricmp(v1, "!active"))
+                op = v1;
 	else if(!stricmp(v1, "-module") || !stricmp(v1, "-installed") || !stricmp(v1, "-has"))
 		op = "-module";
 	else if(!stricmp(v1, "!module") || !stricmp(v1, "!installed"))
@@ -412,6 +426,135 @@ bool ScriptMethods::conditionalExpression(void)
 		return true;
 	}
 
+	if(!stricmp(op, "!expired"))
+	{
+		sym = mapSymbol(v2, 0);
+		if(!sym)
+			return false;
+
+		if(sym->type != symTIMER)
+			return false;
+
+		if(!sym->data[0])
+			return true;
+
+		time(&now);
+		if((long)now >= atol(sym->data))
+			return false;
+		return true;
+	}
+
+	if(!stricmp(op, "-expired"))
+	{
+		sym = mapSymbol(v2, 0);
+		if(!sym)
+			return false;
+
+		if(sym->type != symTIMER)
+			return false;
+
+		if(!sym->data[0])
+			return false;
+
+		time(&now);
+		if((long)now >= atol(sym->data))
+			return true;
+
+		return false;
+	}
+
+
+
+	if(!stricmp(op, "-modify") || !stricmp(op, "!const"))
+	{
+		sym = mapSymbol(v2, 0);
+		if(!sym)
+			return false;
+
+		switch(sym->type)
+		{
+		case symLOCK:
+		case symCONST:
+		case symSEQUENCE:
+			return false;
+		}
+		return true;
+	}
+
+        if(!stricmp(op, "!modify") || !stricmp(op, "-const"))
+        {
+                sym = mapSymbol(v2, 0);
+                if(!sym)
+                        return true;
+
+                switch(sym->type)
+                {
+                case symLOCK:  
+                case symCONST:
+                case symSEQUENCE:
+                        return true;
+                }
+                return false;
+        }  
+
+        if(!stricmp(op, "-queue"))
+        {
+                sym = mapSymbol(v2, 0);
+                if(!sym)
+                        return false;
+
+                switch(sym->type)
+                {
+                case symSTACK:
+		case symFIFO:
+                        return true;
+                }
+                return false;
+        }
+
+        if(!stricmp(op, "!queue"))
+        {
+                sym = mapSymbol(v2, 0);
+                if(!sym)
+                        return false;
+
+                switch(sym->type)
+                {
+                case symFIFO:
+		case symSTACK:
+                        return false;
+                }
+                return true;
+        }
+
+	if(!stricmp(op, "-array"))
+	{
+                sym = mapSymbol(v2, 0);
+                if(!sym)
+                        return false;
+
+		switch(sym->type)
+		{
+		case symARRAY:
+			return true;
+		}
+		return false;
+	}
+
+        if(!stricmp(op, "!array"))  
+        {
+                sym = mapSymbol(v2, 0);
+                if(!sym)
+                        return true;
+
+                switch(sym->type)
+                {
+                case symARRAY:
+                        return false;
+                }
+                return true;
+        }   
+   
 	if(!stricmp(op, "-defined"))
 	{
 		sym = mapSymbol(v2, 0);
@@ -428,6 +571,88 @@ bool ScriptMethods::conditionalExpression(void)
 		return false;
 	}
 
+	if(!stricmp(op, "-active"))
+	{
+		sym = mapSymbol(v2, 0);
+		if(!sym)
+			return false;
+
+		a = (Array *)&sym->data;
+		switch(sym->type)
+		{
+		case symARRAY:
+			if(!a->tail)
+				return false;
+			return true;
+		case symSTACK:
+		case symFIFO:
+			if(a->tail == a->head)
+				return false;
+			return true;
+		case symLOCK:
+			if(strchr(sym->data, ':'))
+				return true;
+			return false;
+		case symCONST:
+		case symSEQUENCE:
+		case symNORMAL:
+		case symPROPERTY:
+		case symORIGINAL:
+		case symMODIFIED:
+		case symCOUNTER:
+			return true;
+		case symINITIAL:
+			return false;
+		case symTIMER:
+		case symNUMBER:
+			if(!sym->data[0])
+				return false;
+		}
+		return true;
+	}
+
+	if(!stricmp(op, "!active"))
+	{
+		sym = mapSymbol(v2, 0);
+		if(!sym)
+			return false;
+
+		a = (Array *)&sym->data;
+		switch(sym->type)
+		{
+		case symARRAY:
+			if(!a->tail)
+				return true;
+			return false;
+		case symSTACK:
+		case symFIFO:
+			a = (Array *)&sym->data;
+			if(a->tail == a->head)
+				return true;
+			return false;
+		case symLOCK:
+			if(strchr(sym->data, ':'))
+				return false;
+			return true;
+		case symCONST:
+		case symSEQUENCE:
+		case symNORMAL:
+		case symPROPERTY:
+		case symORIGINAL:
+		case symMODIFIED:
+		case symCOUNTER:
+			return false;
+		case symINITIAL:
+			return true;
+		case symTIMER:
+		case symNUMBER:
+			if(!sym->data[0])
+				return true;
+		}
+		return false;
+	}
+
+
 	if(!stricmp(op, "-empty"))
 	{
 		v2 = getContent(v2);
@@ -610,7 +835,7 @@ bool ScriptMethods::conditionalExpression(void)
 	return false;
 }
 
-bool ScriptMethods::conditional(void)
+bool ScriptInterp::conditional(void)
 {
 	Line *line = getLine();
 	const char *joiner;
