@@ -837,6 +837,8 @@ bool ScriptMethods::scrTimer(void)
 	Symbol *sym;
 	time_t now;
 	const char *cp;
+	const char *off = getKeyword("offset");
+	const char *exp = getKeyword("expires");
 
 	time(&now);
 
@@ -857,6 +859,12 @@ err:
 set:
 		snprintf(sym->data, 12, "%ld", now);
 		sym->type = symTIMER;
+		if(off)
+			commit(sym, off);
+		else if(exp && atol(exp) > 0)
+			commit(sym, exp);
+		else if(exp)
+			sym->data[0] = 0;
 	}
 	if(err)
 		error(err);
@@ -1247,6 +1255,8 @@ bool ScriptMethods::scrDefine(void)
 		{
 			pv = line->args[idx++];
 			++cp;
+			if(*pv == '{')
+				++pv;
 		}
 		
 		if(prefix && !strchr(cp, '.') && *cp != '%' && *cp != '&')
@@ -1273,6 +1283,9 @@ bool ScriptMethods::scrDefine(void)
 		else
 			vlen = len;
 
+		if(!vlen)
+			++vlen;
+
                 sym = mapSymbol(var, vlen);
 		if(!sym || sym->type != symINITIAL)
 			continue;
@@ -1828,7 +1841,7 @@ bool ScriptMethods::scrEnd(void)
 
 bool ScriptMethods::scrCase(void)
 {
-	unsigned short loop = 0xffff;
+	unsigned short loop = 0xfffe;
 	Line	*line;
 
 	if(!frame[stack].caseflag)
