@@ -72,6 +72,7 @@ size_t Script::pagesize = 1024;
 unsigned Script::symsize = 64;
 unsigned Script::symlimit = 960;
 char Script::decimal = '.';
+bool Script::use_definitions = false;
 bool Script::use_macros = false;
 bool Script::use_prefix = false;
 bool Script::use_merge = false;
@@ -319,6 +320,7 @@ void Script::clear(Symbol *sym)
 		sym->data[1] = 0;
 		return;	
 	case symTIMER:
+		sym->data[0] = 0;
 		return;				
 	}
 }
@@ -394,8 +396,13 @@ const char *Script::extract(Symbol *sym)
 	case symNUMBER:
 		return sym->data;
 	case symTIMER:
-		time(&now);
-		snprintf(sym->data + 12, 12, "%ld",  now - atol(sym->data));
+		if(sym->data[0])
+		{
+			time(&now);
+			snprintf(sym->data + 12, 12, "%ld",  now - atol(sym->data));
+		}
+		else
+			setString(sym->data + 12, 12, "0");
 		return sym->data + 12;
 	case symCOUNTER:
 		value = atoi(sym->data);
@@ -478,6 +485,7 @@ bool Script::commit(Symbol *sym, const char *value)
 	long v1, v2;
 	char vbuf[12];
 	ScriptProperty *p;
+	time_t now;
 
 	if(!sym)
 		return false;
@@ -576,7 +584,13 @@ bool Script::commit(Symbol *sym, const char *value)
 		setString(sym->data + sizeof(Array) + (pos * len), len,	value);
 		return true;
 	case symTIMER:
-		val = atol(sym->data) + atol(value);
+		if(sym->data[0] == 0)
+		{
+			time(&now);
+			val = (long)now + atol(value);
+		}
+		else
+			val = atol(sym->data) + atol(value);
 		snprintf(sym->data, sym->size + 1, "%ld", val);
 		return true;
 	case symCOUNTER:
