@@ -359,6 +359,10 @@ int main(int argc, char **argv)
 
 	slog("testscript", Slog::classUser, Slog::levelInfo);
 
+	shCompiler.compileDefinitions("script.def");
+
+	_image.aliasModule("str", "string");
+
 	name = argv[1];
 	shCompiler.getScript(name);
 	
